/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.file.storage;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.LinkedHashMap;
import java.util.Objects;
import org.json.JSONObject;
import com.openexchange.exception.OXException;
import com.openexchange.guard.api.GuardApi;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.session.Session;

/**
 * {@link GuardDecryptringInputStream} decrypts data using OX Guard
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.2
 */
public class GuardDecryptringInputStream extends AbstractGuardInputStream {

    private final GuardApi guardApi;
    private final GuardAuthenticationToken authToken;
    private final InputStream encrypted;
    private final Session session;

    /**
     * Initializes a new {@link GuardDecryptringInputStream}.
     *
     * @param guardApi The guard enpoint to use
     * @param authToken The authentication token for accessing Guard
     * @param encrypted The data to decrypt
     */
    public GuardDecryptringInputStream(GuardApi guardApi, GuardAuthenticationToken authToken, InputStream encrypted, Session session) {
        this.guardApi = Objects.requireNonNull(guardApi,"guardApi must not be null");
        this.authToken = Objects.requireNonNull(authToken, "authToken must not be null");
        this.encrypted = Objects.requireNonNull(encrypted, "encrypted must not be null");
        this.session = session;
    }

    /**
     * Creates the InputStream which reads the decrypted data from Guard
     *
     * @return An InputStream containing the decrypted data obtained from Guard
     * @throws OXException
     */
    @Override
    protected InputStream getGuardStream() throws OXException {
        ByteArrayInputStream jsonStream = null;
        JSONObject json = new JSONObject();
        JSONObject user = new JSONObject();
        user.putSafe("session", authToken.getGuardSessionId());
        user.putSafe("auth", authToken.getValue());
        json.putSafe("user", user);
        jsonStream = new ByteArrayInputStream(json.toString().getBytes(StandardCharsets.UTF_8));

        LinkedHashMap<String, InputStream> multipartParameters = new LinkedHashMap<>();
        multipartParameters.put("json", jsonStream);
        multipartParameters.put("data", encrypted);

        return guardApi.processResources(GuardApis.mapFor("action", "decrypt", "respondWithJSON", "true" /* forcing guard to render errors in JSON */), multipartParameters, null, session);
    }
}
