/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.transport.listener;

import javax.mail.Address;
import javax.mail.internet.MimeMessage;
import com.openexchange.exception.OXException;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.AbstractGuardAccess;
import com.openexchange.guard.internal.GuardApiImpl;
import com.openexchange.guard.internal.GuardResult;
import com.openexchange.guard.internal.authentication.AuthenticationTokenHandler;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.guard.mail.autocrypt.BackendAutoCryptService;
import com.openexchange.guard.osgi.Services;
import com.openexchange.mail.dataobjects.SecuritySettings;
import com.openexchange.mail.transport.listener.MailTransportListener;
import com.openexchange.mail.transport.listener.Reply;
import com.openexchange.mail.transport.listener.Result;
import com.openexchange.server.ServiceExceptionCode;
import com.openexchange.server.ServiceLookup;
import com.openexchange.session.Session;
import com.openexchange.user.User;
import com.openexchange.user.UserService;

/**
 * {@link GuardTransportListener}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.0
 */
public class GuardTransportListener extends AbstractGuardAccess implements MailTransportListener {

    private static ServiceLookup serviceLookup;

    /**
     * Initializes a new {@link GuardTransportListener}.
     */
    public GuardTransportListener(ServiceLookup serviceLookup) {
        super();
        this.serviceLookup = serviceLookup;
    }

    private User getUserFrom(Session session) throws OXException {
        UserService userService = Services.getService(UserService.class);
        if(userService == null) {
            throw ServiceExceptionCode.SERVICE_UNAVAILABLE.create(UserService.class);
        }
        return userService.getUser(session.getUserId(), session.getContextId());
    }

    @Override
    public boolean checkSettings(SecuritySettings securitySettings, Session session) throws OXException {
        return true;
    }

    @Override
    public Result onBeforeMessageTransport(MimeMessage message, Address[] recipients, SecuritySettings securitySettings, Session session) throws OXException {

        if (securitySettings == null ||
            (!securitySettings.isEncrypt() && !securitySettings.isSign())) {
            BackendAutoCryptService autoCrypt = serviceLookup.getService(BackendAutoCryptService.class);
            if (autoCrypt != null) {
                return new GuardResult(autoCrypt.updateOutgoing(message, session), recipients, Reply.NEUTRAL);

            }
            // Nothing set.  Nothing to do
            return new GuardResult(message, recipients, Reply.NEUTRAL);
        }

        GuardApiImpl guardApi = getGuardApi(PGP_MAIL_ENDPOINT);
        if (null == guardApi) {
            // Guard end point not available
            if (securitySettings.isEncrypt()) { // If set to encrypt, need to fail here
                throw GuardExceptionCodes.MISSING_ENDPOINT.create();
            }
            return new GuardResult(message, recipients, Reply.NEUTRAL);
        }

        GuardAuthenticationToken authToken = new AuthenticationTokenHandler().getForSession(session);
        if(securitySettings.getAuthentication() != null && !securitySettings.getAuthentication().isEmpty()) {
            authToken = GuardAuthenticationToken.fromString(securitySettings.getAuthentication());
        }

        if (securitySettings.isSign()) {  // If signing, Auth code is required
            if (authToken == null) {
                throw GuardExceptionCodes.MISSING_AUTH.create();
            }
        }

        final User user = getUserFrom(session);
        GuardResult processed = guardApi.processMimeMessage(message, recipients, securitySettings,
                 GuardApis.mapFor("action", "process_mime",
                     "user", Integer.toString(session.getUserId()),
                     "context", Integer.toString(session.getContextId()),
                     "fromName", user.getDisplayName(),
                     "isGuest", Boolean.toString(user.isGuest()),
                     "email", user.getMail()
                     ),
                 session,
                 authToken);

        return processed;

    }

    @Override
    public void onAfterMessageTransport(MimeMessage message, Exception exception, Session session) throws OXException {
        // Nothing to do
    }

    // -----------------------------------------------------------------------------------------------------------------------------

}
